@@ -39,8 +39,9 @@ class NFLStatsQueries():
 
     def _generate_pbp_query(self):
         exceptionList = [
-            'description',
-            'interceptedAtPosition', 'penalty', 'passedFromPosition',
+            'description', 'blockingPlayer', 'penalty', 'recoveringTeam',
+            'homePlayersOnField', 'awayPlayersOnField', 'penalties', 'subPlays', 'teamReferences',
+            'interceptedAtPosition', 'passedFromPosition',
         ]
         pbp_schema = self.bqu.get_table_schema(NFL_DATA_DATASET, NFL_PBP_TABLEID)
         fieldsList = []
@@ -59,6 +60,7 @@ class NFLStatsQueries():
                 fieldname = calc_fieldname(name, parent)
                 if field['type'] == 'RECORD':
                     if field['mode'] == 'REPEATED':
+                        print(name)
                         continue
                     else:
                         aggregate_fieldslist(field['fields'], fieldname)
@@ -143,11 +145,11 @@ class NFLStatsQueries():
 
         return 'True'
 
-    def _save_trace_df(self, traceDF, initialColumns, spreadId, sheetName):
+    def _save_trace_df(self, traceDF, initialColumns, spreadId=None, sheetName=None):
         if not spreadId:
             spreadId = '1Q5O3ejSyEDZrlFXX04bOIWOqMxfiHJYimunZKtpFswU'
         if not sheetName:
-            sheetName = 'Trace'
+            sheetName = 'trace'
 
         finalColumns=['season', 'gameid', 'gamename', 'homeScore', 'awayScore','quarter', 'playType', 'currentDown']
 
@@ -168,16 +170,20 @@ class NFLStatsQueries():
             spread.df_to_sheet(traceDF[finalColumns], index=False, sheet=sheetName, start='A1', replace=True)
             print('trace can be found in this url:', spread.url)
         else:
-            traceDF[finalColumns].to_csv('{}.csv'.format(sheetName))
+            fileName = '{}.csv'.format(sheetName)
+            traceDF[finalColumns].to_csv(fileName)
+            print ('Trace to file ', fileName)
             try:
                 import google.colab
                 IN_COLAB = True
                 from google.colab import files
-                files.download('{}.csv'.format(sheetName))
-            except:
+                print ('Downloading', fileName)
+                files.download(fileName)
+            except Exception as e:
+                print ('Error getting trace file {}, {}'.format(fileName, e))
                 IN_COLAB = False
 
-    def pbp_get_stat(self, statName, object, playDimention='all', gameDimention='game', aggfunc=None, playType=None, filter='True', trace=False, traceFileName=None):
+    def pbp_get_stat(self, statName, object, playDimention='all', gameDimention='game', aggfunc=None, playType=None, filter='True', trace=False):
         assert statName in self.statsDict, "Illegal statName: '{}', must be one of {}".format(statName, self.statsDict.keys())
         assert object in self.ptmapDict, "Illegal object: '{}', must be one of {}".format(object, self.ptmapDict.keys())
         #assert playType in self.statsDict, "Illegal playType: '{}', must be one of {}".format(statName, self.statsDict.keys())
@@ -195,9 +201,10 @@ class NFLStatsQueries():
             'gamecond': self._get_gamedimention_condition(gameDimention),
             'playcond': self._get_playdimention_condition(playDimention),
             'filtercond': filter,
+            'isInplayCond': "(df.playType=='penalty') | (df.isNoPlay!=True)"
         }
         df = self.pbpDF
-        queryeval = 'df[({statcond}) & ({gamecond}) & ({playcond}) & ({filtercond})]'.format(**queryInst)
+        queryeval = 'df[({statcond}) & ({gamecond}) & ({playcond}) & ({filtercond}) & ({isInplayCond})]'.format(**queryInst)
         try:
             filteredDF = eval(queryeval)
             #print(filteredDF.shape, df.shape)
@@ -233,7 +240,7 @@ class NFLStatsQueries():
             raise e
 
         if trace:
-            self._save_trace_df(filteredDF, finalDF.columns, sheetName=traceFileName)
+            self._save_trace_df(filteredDF, finalDF.columns, sheetName='trace-{}-{}-{}-{}'.format(statName, object, playDimention, gameDimention))
         #
         # add the rank - denserank.
         finalDF['rank'] = finalDF[aggField].rank(method='dense', ascending=False)
@@ -331,16 +338,16 @@ def test():
     startTime=dt.now()
     pd.set_option('display.max_columns', 12)
     pd.set_option('display.width', 200)
-    os.environ['CONTENDO_AT_HOME'] = 'y'
+    #os.environ['CONTENDO_AT_HOME'] = 'y'
     os.environ["GOOGLE_APPLICATION_CREDENTIALS"] = "{}/sportsight-tests.json".format(os.environ["HOME"])
+    os.environ['CONTENDO_DEV']='y'
     os.chdir('{}/tmp/'.format(os.environ["HOME"]))
     generator = NFLStatsQueries()
     #print(generator._generate_pbp_query())
     print('Start updating pbp data', dt.now() - startTime)
     #generator.update_pbp_data()
-    #generator.read_pbp_data()
     print('Start querying data', dt.now() - startTime)
-    df = generator.pbp_get_stat('yardsRushed', 'rushingPlayer', 'all', 'game',aggfunc='sum', filter="df.season=='2019-regular'", trace=True)
+    df = generator.pbp_get_stat('sacks', 'soloTacklingPlayer', 'all', 'game',aggfunc='count', filter="df.season=='2019-regular'", trace=True)
     #df = generator.pbp_get_composed_stat('yardsPerCatch', 'passingPlayer')
     #df = generator.get_games()
     print(df.columns, df.shape,'\n', df)
